/*
  Author : Benoit PAPILLAULT <benoit.papillault@free.fr>
  Creation: 2004/11/08
  License: GPL

  This program is intented to be run with and without installwatch
*/

#include <stdio.h>
#include <syslog.h>
#include <errno.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/param.h>

void check_syslog()
{
  syslog(LOG_DEBUG, "%s", "installwatch-check");

  if (errno == 0) {
    printf("syslog test failure : please stop syslog during this test\n");
  } else {
    printf("syslog test success\n");
  }
}

void check_open()
{
  int fd;

  errno = 0;
  fd = open("/tmp/toto",O_RDWR|O_CREAT,0666);
  if (fd < 0) {
    perror("open");
  } else {
    if (errno == 0) {
      printf("open test success\n");
    } else {
      printf("open test failure (errno = %d)\n",errno);
    }
    close (fd);
  }
}

/* create a file which a long name */

void check_maxpathlen()
{
  char path1[MAXPATHLEN];
  int i, fd;

  for (i=0;i<MAXPATHLEN-1;i++)
    path1[i] = 'a';
  path1[MAXPATHLEN-1] = 0;

  if ((fd=open(path1,O_RDWR|O_CREAT,0777)) < 0) {
    if (errno != ENAMETOOLONG) {
      perror("open");
    } else {
      printf("MAXPATHLEN test success\n");
    }
  } else {
    close (fd);
    if (unlink(path1) < 0) {
      perror("unlink");
    }
  }
}

void check_syslog_buffer()
{
  const char filename[]="%s%s%s%s";
  int fd;

  /* open + installwatch should crash */
  if ((fd = open(filename,O_RDWR|O_CREAT,0666)) < 0) {
    perror("open");
  } else {
    close (fd);
    if (unlink(filename) < 0) {
      perror("unlink");
    }
  }

  printf("syslog buffer test success\n");
}

/* check that chown/lchown is working properly on symlinks.

  we create a file b.
  we create a symlink a -> b.
  chown must change owner of b
  chown must change owner of a
*/

void check_chown_lchown()
{
  struct stat statbuf;
  int fd;

  if ((fd=open("b",O_RDWR|O_CREAT,0666)) < 0) {
    perror("open b");
    return;
  } else {
    close (fd);
  }

  if (symlink("b","a") < 0) {
    perror("symlink a->b");
  } else {

    /* check chown */

    if (chown ("a", 1, 1) < 0) {
      perror("chown");
    } else {
      if (stat("a",&statbuf) < 0) {
	perror("stat");
      } else {
	if (statbuf.st_uid != 1 || statbuf.st_gid != 1) {
	  printf("chown test failure\n");
	} else {
	  printf("chown test success\n");
	}
      }
    }

    /* check lchown */

    if (lchown ("a", 2, 2) < 0) {
      perror("lchown");
    } else {
      if (lstat("a",&statbuf) < 0) {
	perror("lstat");
      } else {
	if (statbuf.st_uid != 2 || statbuf.st_gid != 2) {
	  printf("lchown test failure\n");
	} else {
	  printf("lchown test success\n");
	}
      }
    }

    /* delete the symlink */
    if (unlink("a") < 0) {
      perror("unlink a");
    }
  }

  if (unlink("b") < 0) {
    perror("unlink b");
  }
}

int main()
{

  check_syslog();
  check_open();
  check_chown_lchown();
  check_maxpathlen();
  check_syslog_buffer();

  return 0;
}
